#ifndef FPUspi
#define FPUspi
#include "FPUdefs.bas"

'==============================================================================
' Copyright  Micromega Corporation 2002-2007. All rights reserved.
'
' @file		FPUspi.bas
' @target:	ARMmite/ARMexpress
'
' This file contains support code for using the uM-FPU V3 floating point
' coprocessor with the ARMexpress or ARMmite over an SPI interface.
'
' @author Cam Thompson, Micromega Corporation
' @version
'	March 31, 2008
'	- updated for ARMbasic V7
'   August 13, 2007
'   - use #define for FpuCS definition
'	August 2, 2007
'	- original version
'==============================================================================

#ifndef USE_SPI
  #define SPIpreSample 1
  #include <SPI.bas>
#else
  #ifndef SPIpreSample
    #error SPIpreSample must be defined before #include <SPI.bas>
  #endif
#endif

'==============================================================================
'-------------------- uM-FPU V3.1 SPI definitions -----------------------------
'==============================================================================

const	FpuClk		= 15		' SPI clock	   (connects to uM-FPU SCLK/SCL pin)
const	FpuOut		= 14		' SPI data out (connects to uM-FPU SIN/SDA pin)
const	FpuIn		= 13		' SPI data in  (connects to uM-FPU SOUT pin)
#define	FpuCS		-1			' SPI CS	   (connects to uM-FPU CS pin)
								' Note: set FpuCS to -1 unless uM-FPU mode has
								' been selected to use CS pin as chip select

'==============================================================================
'-------------------- uM-FPU V3.1 SPI support routines ------------------------
'==============================================================================

SUB Fpu_Wait
  #if FpuCS >= 0
	OUT(FpuCS) = 0
  #endif
	INPUT(FpuIn)
	while (IN(FpuIn))
	loop
  #if FpuCS >= 0
	OUT(FpuCS) = 1
  #endif
END SUB

SUB Fpu_Write(bval1)
	str$(0) = bval1
	SPIOUT(FpuCS, FpuClk, FpuOut, 0, 1, str$)
END SUB

SUB Fpu_Write2(bval1, bval2)
	str$(0) = bval1
	str$(1) = bval2
	SPIOUT(FpuCS, FpuClk, FpuOut, 0, 2, str$)
END SUB

SUB Fpu_Write3(bval1, bval2, bval3)
	str$(0) = bval1
	str$(1) = bval2
	str$(2) = bval3
	SPIOUT(FpuCS, FpuClk, FpuOut, 0, 3, str$)
END SUB

SUB Fpu_Write4(bval1, bval2, bval3, bval4)
	str$(0) = bval1
	str$(1) = bval2
	str$(2) = bval3
	str$(3) = bval4
	SPIOUT(FpuCS, FpuClk, FpuOut, 0, 4, str$)
END SUB

SUB Fpu_Write5(bval1, bval2, bval3, bval4, bval5)
	str$(0) = bval1
	str$(1) = bval2
	str$(2) = bval3
	str$(3) = bval4
	str$(4) = bval5
	SPIOUT(FpuCS, FpuClk, FpuOut, 0, 5, str$)
END SUB

SUB Fpu_Write6(bval1, bval2, bval3, bval4, bval5, bval6)
	str$(0) = bval1
	str$(1) = bval2
	str$(2) = bval3
	str$(3) = bval4
	str$(4) = bval5
	str$(5) = bval6
	SPIOUT(FpuCS, FpuClk, FpuOut, 0, 6, str$)
END SUB

SUB Fpu_Write7(bval1, bval2, bval3, bval4, bval5, bval6, bval7)
	str$(0) = bval1
	str$(1) = bval2
	str$(2) = bval3
	str$(3) = bval4
	str$(4) = bval5
	str$(5) = bval6
	str$(6) = bval7
	SPIOUT(FpuCS, FpuClk, FpuOut, 0, 7, str$)
END SUB

SUB Fpu_Write8(bval1, bval2, bval3, bval4, bval5, bval6, bval7, bval8)
	str$(0) = bval1
	str$(1) = bval2
	str$(2) = bval3
	str$(3) = bval4
	str$(4) = bval5
	str$(5) = bval6
	str$(6) = bval7
	str$(7) = bval8
	SPIOUT(FpuCS, FpuClk, FpuOut, 0, 8, str$)
END SUB

SUB Fpu_WriteWord(wval)
	str$(0) = wval>>8
	str$(1) = wval
	SPIOUT(FpuCS, FpuClk, FpuOut, 0, 2, str$)
END SUB

SUB Fpu_WriteLong(lval)
	str$(0) = lval>>24
	str$(1) = lval>>16
	str$(2) = lval>>8
	str$(3) = lval
	SPIOUT(FpuCS, FpuClk, FpuOut, 0, 4, str$)
END SUB

SUB Fpu_WriteFloat(fval)
	Fpu_WriteLong(fval)
END SUB

SUB Fpu_WriteString(BYREF s as string)
	SPIOUT(FpuCS, FpuClk, FpuOut, 0, len(s)+1, s)
END SUB
	
SUB Fpu_ReadDelay
'	enough delay provided by call overheads
'	time1 = TIMER
'	while ((TIMER - time1) < 15)
'	loop
END SUB

FUNCTION Fpu_Read
	Fpu_ReadDelay
	str$(0) = 0
	SPIIN(FpuCS, FpuIn, FpuClk, FpuOut, 0, 0, str$, 1, str$)
	return str$(0)
END FUNCTION

FUNCTION Fpu_ReadWord
	Fpu_ReadDelay
	str$(0) = 0
	SPIIN(FpuCS, FpuIn, FpuClk, FpuOut, 0, 0, str$, 2, str$)
	return (str$(0)<<8) + str$(1)
END FUNCTION

FUNCTION Fpu_ReadLong
	Fpu_ReadDelay
	str$(0) = 0
	SPIIN(FpuCS, FpuIn, FpuClk, FpuOut, 0, 0, str$, 4, str$)
	return (str$(0)<<24) + (str$(1)<<16) + (str$(2)<<8) + str$(3)	
END FUNCTION

FUNCTION Fpu_ReadFloat
	return Fpu_ReadLong	
END FUNCTION

FUNCTION Fpu_ReadString(opcode) as string
	Fpu_Wait
	if opcode <> READSEL then opcode = READSTR
	Fpu_Write(opcode)
	gosub FPU_ReadDelay
	str$(0) = 0
	SPIIN(FpuCS, FpuIn, FpuClk, FpuOut, 0, 0, str$, 0, str$)
	return str$
END FUNCTION

FUNCTION Fpu_ReadStatus
	Fpu_Wait
	Fpu_Write(READSTATUS)
    return Fpu_Read
END FUNCTION

FUNCTION Fpu_Reset
	dim i as integer
	for i = 1 to 10
		Fpu_Write($FF)
	next i
	Fpu_Write(0)
	wait(10)
	Fpu_Write(SYNC)
	return Fpu_Read
END FUNCTION
	
SUB Print_Version
	Fpu_Write(VERSION)
	print Fpu_ReadString(READSTR);
END SUB

SUB Print_Float(format)
	Fpu_Write2(FTOA, format)
	print Fpu_ReadString(READSTR);
END SUB

SUB Print_Long(format)
	Fpu_Write2(LTOA, format)
	print Fpu_ReadString(READSTR);
END SUB

SUB Print_FpuString(opcode)
	print Fpu_ReadString(opcode);
END SUB

#endif